
$(document).ready(() => {

    toggleLoading(true);

    // Search Bar Functionality

    searchTimeout = null;

    $("#search").on("input", () => {
        clearTimeout(searchTimeout);
        toggleLoading(true);

        searchTimeout = setTimeout(() => {
            fetchAndLoadGroups();
        }, 500)
    });

    $("#searchButton").on("click", () => {
        fetchAndLoadGroups();
    });

    $("#sortForm input").on("click", function() {
        const name = $(this).attr("name");
        localStorage.setItem(name, $(`input[name='${name}']:checked`, "#sortForm").val());

        fetchAndLoadGroups();
    });

    // Modals

    $("#addAngler").on("click", () => {
        addAngler(-1);
    });

    $("#addTeam").on("click", () => {
        addTeam(-1);
    });

    $("#addSection").on("click", () => {
        addSection(-1);
    });

    fetchAndLoadGroups();

});

function fetchAndLoadGroups() {
    toggleLoading(true);

    const showGroups = localStorage.getItem("showGroups");
    const sortAnglers = localStorage.getItem("sortAnglers");
    const task = showGroups === "teams" ? "get-teams" : "get-sections";

    $.ajax({
        type: "POST",
        url: pageUrl,
        data: {
            csrfmiddlewaretoken: csrfMiddlewareToken,
            tasks: [task, "get-anglers"],
            sortAnglers: sortAnglers,
            search: $("#search").val()
        },
        error: (error) => {
            console.error(error);
        },
        success: (data) => {
            loadGroups(data);
        }
    });

    toggleLoading(false);
}

function groupTemplate(group, groupType, oppositeGroupType) {
    groupValue = groupType === "teams" ? group.number : group.character

    return `
        <div class='col-12 col-md-6 col-xl-4 mb-4'>
            <div
                class='${groupType.toLowerCase()} px-4 py-3 bg-body-tertiary bg-gradient rounded h-100 fluid-hover-zoom shadow-sm md-shadow-on-hover'
                data-value='${groupValue}'>
                <h3>
                    <span class='fs-4'>
                        <span class='fs-6'>${groupType.toUpperCase()}</span> 
                        ${groupValue}
                    </span>
                    <button class='btn btn-sm btn-light float-end border-0 me-1'>
                        <i class='bi bi-gear fs-6'></i>
                    </button>
                </h3>
                <ul class='list-unstyled anglers mt-3'>
                </ul>
            </div>
        </div>
    `
}

function anglerTemplate(angler, oppositeGroupType) {
    const fullname = angler.first_name + " " + angler.last_name;
    const oppositeGroupValue = oppositeGroupType === "teams"? angler.team_number : angler.section_character;

    var oppositeGroup = oppositeGroupType.slice(0, -1);
    oppositeGroup = oppositeGroup.charAt(0).toUpperCase() + oppositeGroup.slice(1);

    return `
        <li class='mb-3 rounded w-100 fluid-hover-zoom'>
            <div 
                class='angler d-flex'
                data-first='${angler.first_name}'
                data-last='${angler.last_name}'
                data-id='${angler.id}'
                data-peg-number='${angler.peg_number}'
                data-team-id='${angler.team_id}'
                data-section-id='${angler.section_id}'>
                <div class='px-3 py-2 w-100'>
                    <div class='badge bg-secondary-subtle text-body fixed-badge' data-bs-title='Peg ${angler.peg_number}' data-bs-toggle='tooltip'>
                        ${angler.peg_number}
                    </div>
                    <div class='badge bg-secondary-subtle text-body fixed-badge' data-bs-title='${oppositeGroup} ${oppositeGroupValue}' data-bs-toggle='tooltip'>
                        ${oppositeGroupValue}
                    </div>
                    <div class='d-inline-block ms-2 angler-fullname'>
                        ${fullname}
                    </div>
                </div>
                <button type='button' class='ms-auto btn btn-light border-0 fs-6 pencil-btn align-self-center me-1 force-contents-center'>
                    <i class='bi bi-pencil'></i>
                </button>
            </div>
        </li>
    `
}

function loadGroups(data) {
    const showGroups = localStorage.getItem("showGroups");
    const groupType = showGroups === "teams" ? "teams" : "sections";
    const oppositeGroupType = showGroups === "teams" ? "sections" : "teams";
    const groups = data[groupType];

    $("#groups").html("");

    groups.forEach((group) => {
        $("#groups").append(groupTemplate(group, groupType, oppositeGroupType));
        data.anglers.forEach((angler) => {
            if (angler.team_id === group.id || angler.section_id === group.id) {
                $("#groups").find(".anglers").last().append(
                    anglerTemplate(angler, oppositeGroupType)
                );
            }
        });
    });

    activateTooltips();
}

/**
 * Shows a loading icon, hides groups and error alerts.
 * 
 * @param {Boolean} loading - Wether or not to show the loading icon 
 */
function toggleLoading(loading) {
    if (loading) {
        $("#groups").hide();
        $("#notFound").hide();
        $("#loadingSpinner").show();
    }
    else {
        $("#groups").show();
        $("#loadingSpinner").hide();
    }
}

/**
 * Open the modal for adding/editing an angler
 * 
 * @param {Number} anglerId - ID of the angler, if -1 will create a new angler 
 */
function addAngler(anglerId) {
    $("#anglerModal").modal("show");
}

/**
 * Open the modal for adding/editing a team
 * 
 * @param {Number} teamId - ID of the team, if -1 will create a new team
 */
function addTeam(teamId) {
    $("#teamModal").modal("show");
}

/**
 * Open the modal for adding/editing a section
 * 
 * @param {Number} sectionId - ID of the section, if -1 will create a new section
 */
function addSection(sectionId) {
    $("#sectionModal").modal("show");
}
